// ===============================================================================================
package mahjongg;

import javax.swing.JPanel;
import java.awt.*;
import java.awt.image.*;
import java.awt.Graphics2D.*;
import java.awt.event.*;
import javax.imageio.*;
import java.io.*;
import java.lang.Math;


class spielfeld extends JPanel
{
  public static final long serialVersionUID = 1L;

  final int cmax=36;
  final int sxmax=1280;
  final int symax=960;

  int axmax=800;
  int aymax=600;
  int dx=28;
  int dy=28;
  
  String bildpfad;
  Color color_bg=new Color(0,0,0,128);
  Color color_rahmen=new Color(0,0,96);

  Image bgima;
  Image ima[]=new Image[2*cmax];
  BufferedImage bima=new BufferedImage(sxmax,symax,BufferedImage.TYPE_INT_ARGB);
  Graphics gb=bima.getGraphics();

  int aktscreen=-1;
  main mama;  // parent
  spiel p_game; // Pointer auf Class Spiel
  Font font=new Font("SansSerif",Font.BOLD,14);
  FontMetrics metrics = getFontMetrics(font);

  public spielfeld(main parent)
  { 
    setLayout(new BorderLayout());
    mama=parent;
    p_game=mama.my_spiel;
    init_data();
  }

// slots:
  public void set_screen(int nr) { aktscreen=nr; }

  public void set_background(String bildname)
  { int dx,dy,xx,yy;
    bgima=hole_bild(bildpfad+"backgrounds/"+bildname);
    dx=bgima.getWidth(null);
    dy=bgima.getHeight(null);
    yy=0;
    while (yy<symax) { xx=0; while (xx<sxmax) { gb.drawImage(bgima,xx,yy,null); xx=xx+dx; } yy=yy+dy; }
  }

  public void set_tileset(String bildname) 
  { for (int i=0; i<cmax; i++)
    { ima[i]=hole_bild(bildpfad+"tilesets/"+bildname.replace("#",(""+i)));
      ima[i+cmax]=hole_bild(bildpfad+"tilesets/"+bildname.replace("#",(i+"a")));
    }
  }

  public int get_xmax()
  { int xmax;
    switch(aktscreen)
    { case 0: xmax=axmax; break; // von pattern berechnen lassen
      case 1: xmax=34*p_game.stein[0].dx; break;
      case 2: xmax=640;  break;
      default: xmax=720;
    }
    return xmax;
  }
  public int get_ymax()
  { int ymax;
    switch(aktscreen)
    { case 0: ymax=aymax; break;
      case 1: ymax=22*p_game.stein[0].dy; break;
      case 2: ymax=480;  break;
      default: ymax=520;
    }
    return ymax;
  }

  // ----------------------------------------------------------------------------------------------------

  void init_data()
  { bildpfad="/"+mama.get_kennung()+"/images/";
    set_background("leder.jpg");
    set_tileset("adv_#.gif");
  }

  public void paintComponent(Graphics g)
  { g.setColor(Color.black);
    g.fillRect(0,0,sxmax,symax);
    g.drawImage(bima,0,0,null);
    g.setFont(font);
    switch (aktscreen)
    { case 0: mach_spielfeld(g); break;
      case 1: zeige_reste(g);    break;
      case 2: zeige_hilfe(g);    break;
      default: zeige_titel(g);
    }
    mama.zwischenlager.setPreferredSize(new Dimension(get_xmax(),get_ymax()));
    mama.zwischenlager.revalidate();
  }

  Image hole_bild(String filnam)
  { Image ima=null;
    try { ima=ImageIO.read(getClass().getResource(filnam)); }
    catch(IOException io) { System.out.println(filnam+" not found or valid");   }
    return ima;
  }

  void mach_spielfeld(Graphics g)
  { int i,xx,yy;
    axmax=520;
    aymax=400;
    for (i=0; i<p_game.maxsprites; i++) if (p_game.stein[i].typ>=0)
    { g.drawImage(ima[p_game.stein[i].typ],p_game.stein[i].x,p_game.stein[i].y,null);
      xx=p_game.stein[i].x+3*p_game.stein[i].dx+16;
      yy=p_game.stein[i].y+3*p_game.stein[i].dy+16; 
      if (axmax<xx) axmax=xx;
      if (aymax<yy) aymax=yy;
    }
    g.setColor(color_bg); g.fillRect(8,0,632,28);
    g.setColor(color_rahmen); g.drawRect(8,0,632,28);
    g.setFont(font);
    g.setColor(Color.yellow);
    g.drawString(mama.langdef.get("RTILES")+": "+p_game.rest+"   "+p_game.messi,16,18); 
  }

  void zeige_hilfe(Graphics g)
  { g.setColor(color_bg);
    g.fillRect(16,16,560,400);
    g.setColor(color_rahmen); g.drawRect(16,16,560,400);
    g.setColor(Color.yellow);
    g.drawString(mama.langdef.get("HELP"),64,32);
    g.drawLine(16,48,560,48);
    for (int i=0; i<mama.hilfetext.length; i++) g.drawString(mama.hilfetext[i],32,80+16*i);
  }

  void zeige_titel(Graphics g)
  { g.setColor(color_bg);
    g.fillOval(128,192,256,48);
    g.setColor(color_rahmen);
    g.drawOval(128,192,256,48);
    g.setColor(Color.yellow);
    g.drawString("Start: Ctrl-N",192,208);
    g.drawString(mama.langdef.get("HELP")+": Ctrl-H",192,232);
  }

  void zeige_reste(Graphics g)
  { // steine sortiert nach typ
    int i,k,xx,yy,xmax=0,ymax=0;
    int[] ind=sortiere_sprites_nach_typ();
    g.setColor(color_rahmen); g.drawRect(32,16,sxmax-64,32);
    g.setColor(Color.yellow); g.drawString(mama.langdef.get("RTILES"),64,40);
    xx=0; yy=0;
    for (i=0; i<p_game.maxsprites; i++)
    { k=ind[i];
      if (p_game.stein[k].typ>=0)
      { g.drawImage(ima[p_game.stein[k].typ],xx*2*dx+16,yy*2*dy+64,null);
        xx++;
        if (xx>=16) { xx=0; yy++; }
      }
    }
  }

// Hilfsroutinen: --------------------------------------------------------------------------
  int[] sortiere_sprites_nach_typ() // ujassort
  { boolean getauscht;
    int i,k,delta,max=p_game.maxsprites;
    int[] ind=new int[max];
    for (i=0; i<max; i++) ind[i]=i;
    delta=(int)Math.floor(max/2);
    while (delta>0) 
    { getauscht=false;
      for (i=0; i<(max-delta); i++)
      { if (p_game.stein[ind[i]].typ>p_game.stein[ind[i+delta]].typ) // Sortier-Bedingung
        { k=ind[i];
          ind[i]=ind[i+delta];
          ind[i+delta]=k;
          getauscht=true;
        }
      }
      if (!getauscht) delta=(int)Math.floor(delta/2);
    }
    return ind;
  }



}